<?php

declare(strict_types=1);

namespace Drupal\sites_group\ContextProvider;

use Drupal\Core\Plugin\Context\Context;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\sites\ContextProvider\CurrentSiteContextInterface;
use Drupal\sites\Plugin\Site\SiteInterface;
use Drupal\sites_group\Plugin\Site\GroupSiteInterface;

/**
 * The current_site context provider decorator.
 */
class CurrentSiteContextDecorator implements CurrentSiteContextInterface {

  use StringTranslationTrait;

  /**
   * Constructs a CurrentSiteContextDecorator object.
   *
   * @param \Drupal\sites\ContextProvider\CurrentSiteContextInterface $parent
   *   The current_site context service we decorate.
   */
  public function __construct(
    protected readonly CurrentSiteContextInterface $parent
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getRuntimeContexts(array $unqualified_context_ids) {
    $runtime_contexts = $this->parent->getRuntimeContexts($unqualified_context_ids);

    $site_context = $runtime_contexts['site'];
    if ($site_context->hasContextValue()) {
      $site = $site_context->getContextValue();
      if ($site instanceof GroupSiteInterface) {
        $group = $site->getGroup();
      }
      else {
        // Provide a default value for context mapping selection when not in group context.
        $group_types = \Drupal::service('sites_group.service')->getGroupTypes();
        if (!empty($group_types)) {
          $group_types = array_keys($group_types);
          array_rand($group_types);
          $group_type = reset($group_types);
          $group = \Drupal::entityTypeManager()->getStorage('group')->create(['type' => $group_type]);
        }
      }
      $sites_group = new Context(new EntityContextDefinition('entity:group', $this->t('Group from site')), $group ?? NULL);

      $runtime_contexts['group'] = $sites_group;
    }

    return $runtime_contexts;
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailableContexts() {
    return $this->getRuntimeContexts([]);
  }

  /**
   * {@inheritdoc}
   */
  public function getSiteFromContext(): ?SiteInterface {
    return $this->parent->getSiteFromContext();
  }

  /**
   * {@inheritdoc}
   */
  public function clearActiveSite(): void {
    $this->parent->clearActiveSite();
  }

  /**
   * {@inheritdoc}
   */
  public function isReady(): bool {
    return $this->parent->isReady();
  }

}
