<?php

declare(strict_types=1);

namespace Drupal\sites_group\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;
use Drupal\sites\SitePluginManagerInterface;
use Drupal\sites_group\SitesGroupServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the content push.
 *
 * @todo Cleanup
 */
class SitesGroupSitesForm extends FormBase {

  /**
   * The entity from route.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected $entity = NULL;

  /**
   * Constructs a SitesGroupEntityContentPushForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   *   The route match.
   * @param \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface $groupRelationTypeManager
   *   The group relation type manager.
   * @param \Drupal\sites_group\SitesGroupServiceInterface $sitesGroupService
   *   The sites_group service.
   * @param \Drupal\sites\SitePluginManagerInterface $sitePluginManager
   *   The site plugin manager.
   */
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    RouteMatchInterface $routeMatch,
    protected readonly GroupRelationTypeManagerInterface $groupRelationTypeManager,
    protected readonly SitesGroupServiceInterface $sitesGroupService,
    protected readonly SitePluginManagerInterface $sitePluginManager
  ) {
    $this->routeMatch = $routeMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('group_relation_type.manager'),
      $container->get('sites_group.service'),
      $container->get('plugin.manager.site'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sites_group_sites';
  }

  /**
   * The _title_callback for the sites page.
   *
   * @return string
   *   The page title.
   */
  public function title() {
    return $this->t('Sites')->__toString();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $entity = $this->getEntity();

    foreach ($this->sitesGroupService->getAvailableGroupSiteTypesByEntity($entity) as $group_type_id => $group_type) {
      $form[$group_type_id] = [
        '#type' => 'details',
        '#title' => $group_type->label(),
        '#tree' => TRUE,
        '#open' => TRUE,
      ];

      $header = [
        'group' => $this->t('Site'),
        'operations' => $this->t('Operations'),
      ];

      $form[$group_type_id]['groups'] = [
        '#type' => 'tableselect',
        '#header' => $header,
        '#options' => $this->getTableSelectOptions($group_type)['options'],
        '#default_value' => $this->getTableSelectOptions($group_type)['default_value'],
        '#empty' => $this->t('No entries found for: @group_type. Create one <a href=":group_create_link">here</a>.', [
          '@group_type' => $group_type->label(),
          ':group_create_link' => Url::fromRoute('entity.group.add_form', ['group_type' => $group_type->id()])->toString(),
        ]),
      ];
    }

    $form['actions'] = ['#type' => 'action'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * Builds the table select options for the form.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group type config entity.
   */
  protected function getTableSelectOptions(GroupTypeInterface $group_type): array {
    // Load groups by group type, representing one row in the table.
    $groups = $this->entityTypeManager->getStorage('group')->loadByProperties(['type' => $group_type->id()]);

    // Sort by name for better readability.
    usort($groups, function ($a, $b) {
      return $a->label() <=> $b->label();
    });

    $options = [];
    $selected = [];
    // Init the rows for each group.
    foreach ($groups as $group) {
      // List groups that are sites only.
      if (!$this->sitePluginManager->hasDefinition('sites_group:' . $group->id())) {
        continue;
      }
      $selected[$group->id()] = 0;
      $options[$group->id()] = [
        'group' => $group->label(),
        'operations' => [],
      ];
      if (!$group->hasPermission("create {$this->sitesGroupService->getPluginIdByEntity($this->entity)} relationship", $this->currentUser())) {
        $options[$group->id()]['#disabled'] = TRUE;
      }
    }

    // Check if we have relations in place we can use to provide some
    // operations.
    $relations = $this->entityTypeManager->getStorage('group_relationship')->loadByProperties([
      'entity_id' => $this->entity->id(),
      'plugin_id' => $this->sitesGroupService->getPluginIdByEntity($this->entity),
      'group_type' => $group_type->id(),
    ]);

    // Build operations and track which relations exist to set the default value
    // for the checkboxes of the tableselect element.
    foreach ($relations as $relation) {
      $selected[$relation->getGroup()->id()] = $relation->getGroup()->id();

      $operations = [];
      // @todo Cleanup - override function.
      foreach ($relation->getTranslationLanguages() as $langcode => $language) {
        // @todo Check for access?
        // @todo Implement fallback if we can not masquerade automatically.
        if ($relation->getEntity()->hasTranslation($langcode)) {
          $link = $relation->getEntity()->getTranslation($langcode)->toUrl('edit-form');
          $operations['edit.' . $langcode] = [
            'title' => $this->t('Edit @language', ['@language' => $langcode]),
            'url' => $link,
            'weight' => 10,
          ];
        }
      }

      if ($relation->getGroup()->hasPermission("create {$this->sitesGroupService->getPluginIdByEntity($this->entity)} relationship", $this->currentUser())) {
        $options[$relation->getGroup()->id()]['operations'] = [
          // 'data' => $this->entityTypeManager->getListBuilder($relation->getEntityTypeId())->buildOperations($relation)
          'data' => [
            '#type' => 'operations',
            '#links' => $operations,
            // Allow links to use modals.
            '#attached' => [
              'library' => ['core/drupal.dialog.ajax'],
            ],
          ],
        ];
      }
    }

    return [
      'options' => $options,
      'default_value' => $selected,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $entity = $this->getEntity();
    $plugin_id = $this->sitesGroupService->getPluginIdByEntity($entity);

    foreach ($this->sitesGroupService->getAvailableGroupSiteTypesByEntity($entity) as $group_type_id => $group_type) {
      $changed_groups = array_diff_assoc($form_state->getValue($group_type_id)['groups'], $form[$group_type_id]['groups']['#default_value']);

      /** @var \Drupal\group\Entity\Storage\GroupRelationshipStorageInterface $relation_storage */
      $relation_storage = $this->entityTypeManager->getStorage('group_relationship');

      foreach ($changed_groups as $group_id => $selected) {
        /** @var \Drupal\group\Entity\GroupInterface $group */
        $group = $this->entityTypeManager->getStorage('group')->load($group_id);

        // Don't act on groups the user does not have access to.
        if (!$group->hasPermission("create {$this->sitesGroupService->getPluginIdByEntity($this->entity)} relationship", $this->currentUser())) {
          continue;
        }

        // Create new relationship if selected.
        if ($selected) {
          // @todo Only add langcode if available.
          $group->addRelationship($entity, $plugin_id, ['langcode' => $entity->getUntranslated()->langcode->value]);
        }
        // Remove relationship if not selected.
        else {
          $group_relations = $relation_storage->loadByEntityAndGroup($entity, $group, $plugin_id);
          foreach ($group_relations as $group_relation) {
            $group_relation->delete();
          }
        }
      }
    }
  }

  /**
   * Access callback for the route.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   An access result object.
   */
  public function access(AccountInterface $account) {
    if ($entity = $this->getEntity()) {
      foreach ($this->sitesGroupService->getAvailableGroupSiteTypesByEntity($entity) as $group_type_id => $group_type) {
        $groups = $this->entityTypeManager->getStorage('group')->loadByProperties(['type' => $group_type->id()]);
        foreach ($groups as $group) {
          if ($group->hasPermission("create {$this->sitesGroupService->getPluginIdByEntity($this->entity)} relationship", $account)) {
            return AccessResult::allowed();
          }
        }
      }
    }

    return AccessResult::neutral();
  }

  /**
   * Loads the entity from route.
   *
   * Since we don't know on which entity_type this is used, we cannot get the
   * entity upcasted by the route for us.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface|null
   *   The entity from Route.
   */
  public function getEntity() {
    if (!$this->entity) {
      foreach ($this->routeMatch->getParameters() as $parameter) {
        if ($parameter instanceof ContentEntityInterface) {
          $this->entity = $parameter;
          continue;
        }
      }

      return $this->entity;
    }

    return $this->entity;
  }

}
