<?php

declare(strict_types=1);

namespace Drupal\sites_group\FormDecorator;

use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\form_decorator\EntityFormDecoratorBase;
use Drupal\group\Entity\GroupTypeInterface;

/**
 * Plugin implementation for "group_type_edit_form".
 *
 * @FormDecorator(
 *   hook = "form_group_type_edit_form_alter"
 * )
 */
final class GroupTypeEditFormAlter extends EntityFormDecoratorBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ...$args) {
    $form = parent::buildForm($form, $form_state, ...$args);

    /** @var \Drupal\group\Entity\GroupTypeInterface $group_type */
    $group_type = $this->getEntity();

    $form['sites_group_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Sites group settings'),
      '#description' => $this->t('Configure this group type to be used as sites_group.'),
      '#tree' => TRUE,
      '#open' => $group_type->getThirdPartySetting('sites_group', 'status') ?? FALSE,
    ];

    $form['sites_group_settings']['status'] = [
      '#title' => $this->t('Enable'),
      '#type' => 'checkbox',
      '#default_value' => $group_type->getThirdPartySetting('sites_group', 'status'),
      '#description' => $this->t('Use this group type to create sites.'),
    ];

    // Field mapping configuration.
    $form['sites_group_settings']['field_map'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Field mapping'),
      'description' => [
        '#markup' => '<p>' . $this->t('Sites are required to have metadata fields such as the site name, hostname, etc. You can use entity fields on the group for that, which can be mapped to the metadata below.') . '</p>',
      ],
      '#states' => [
        'visible' => [
          'input[name="sites_group_settings[status]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $field_map = $group_type->getThirdPartySetting('sites_group', 'field_map', []);
    foreach ($this->getFieldMappingsByGroupType($group_type) as $field_name => $element) {
      $form['sites_group_settings']['field_map'][$field_name] = array_merge($element, [
        '#type' => 'select',
        '#options' => $this->getFieldOptionsByGroupType($group_type),
        '#default_value' => $field_map[$field_name] ?? ($element['#default_value'] ?? '_none'),
      ]);
    }

    // Default settings configuration.
    $form['sites_group_settings']['default_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Default settings'),
      'description' => [
        '#markup' => '<p>' . $this->t('For some metadata you probably want to set a default value.') . '</p>',
      ],
      '#states' => [
        'visible' => [
          'input[name="sites_group_settings[status]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $default_settings_values = $group_type->getThirdPartySetting('sites_group', 'default_settings', []);
    foreach ($this->getDefaultSettingsByGroupType($group_type) as $field_name => $element) {
      $form['sites_group_settings']['default_settings'][$field_name] = array_merge($element, [
        '#default_value' => $default_settings_values[$field_name] ?? ($element['#default_value'] ?? NULL),
      ]);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    return $this->inner->validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $group_type = $this->getEntity();

    $sites_group_settings = $form_state->getValue('sites_group_settings');
    $sites_group_settings['field_map'] = array_filter($sites_group_settings['field_map'], function ($field_name) {
      return $field_name != '_none';
    });

    foreach ($sites_group_settings as $key => $sites_group_setting) {
      $group_type->setThirdPartySetting('sites_group', $key, $sites_group_setting);
    }

    $this->inner->setEntity($group_type);
    return $this->inner->save($form, $form_state);
  }

  /**
   * Builds an array of field mapping select fields.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group_type entity.
   *
   * @return array
   *   An array of select form elements partials.
   */
  protected function getFieldMappingsByGroupType(GroupTypeInterface $group_type): array {
    $field_mappings = [
      'name' => [
        '#title' => $this->t('The sites name'),
        '#description' => $this->t('The name of the site.'),
        '#required' => TRUE,
        '#default_value' => 'label',
      ],
      'hostnames' => [
        '#title' => $this->t('Hostnames'),
        '#description' => $this->t('The hostnames of the site. Should be in format "drupal.org" without protocol or trailing slash.'),
        '#required' => FALSE,
      ],
      'theme' => [
        '#title' => t('The theme'),
      ],
      'admin_theme' => [
        '#title' => t('The admin theme'),
      ],
      'front_page' => [
        '#title' => $this->t('Front page'),
        '#description' => $this->t('Specify a relative URL to display as the front page for the site. Skip if you want to use the group site as front page.'),
        '#required' => FALSE,
      ],
      '403_page' => [
        '#title' => $this->t('403 page'),
        '#description' => $this->t('Specify a relative URL to display as the 403 page for the site. Skip if you want to use the global 403 page.'),
        '#required' => FALSE,
      ],
      '404_page' => [
        '#title' => $this->t('404 page'),
        '#description' => $this->t('Specify a relative URL to display as the 404 page for the site. Skip if you want to use the global 404 page.'),
        '#required' => FALSE,
      ],
      'weight' => [
        '#title' => $this->t('Weight'),
        '#description' => $this->t('Specify weight to sort the site.'),
        '#required' => FALSE,
      ],
    ];

    \Drupal::moduleHandler()->alter('sites_group_field_mappings', $field_mappings, $group_type);

    return $field_mappings;
  }

  /**
   * Builds an array of form elements for the default settings.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group_type entity.
   *
   * @return array
   *   An array of form elements.
   */
  protected function getDefaultSettingsByGroupType(GroupTypeInterface $group_type): array {
    $default_settings = [
      'theme' => [
        '#title' => t('The default theme'),
        '#type' => 'select',
        '#options' => $this->getThemesOptions(),
        '#required' => TRUE,
      ],
      'admin_theme' => [
        '#title' => t('The admin theme'),
        '#type' => 'select',
        '#options' => $this->getThemesOptions(),
        '#required' => TRUE,
      ],
    ];
    \Drupal::moduleHandler()->alter('sites_group_default_settings', $default_settings, $group_type);

    return $default_settings;
  }

  /**
   * Builds an options array for a group_type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group_type entity.
   *
   * @return array
   *   An options array.
   */
  protected function getFieldOptionsByGroupType(GroupTypeInterface $group_type): array {
    $options = ['_none' => $this->t('- Skip setting -')];
    foreach (\Drupal::service('entity_field.manager')->getFieldDefinitions($group_type->getEntityType()->getBundleOf(), $group_type->id()) as $field_name => $field) {
      if (!($field instanceof BaseFieldDefinition) || $field_name === 'name') {
        $options[$field_name] = $field->getLabel();
      }
    }

    natcasesort($options);
    return $options;
  }

  /**
   * Builds an options array for the default theme selection.
   *
   * @return array
   *   An options array.
   */
  protected function getThemesOptions() {
    $theme_options = [];
    $themes = \Drupal::service('theme_handler')->listInfo();
    foreach ($themes as $theme_name => $theme_info) {
      if (!empty($theme_info->status)) {
        $theme_options[$theme_name] = $theme_info->info['name'];
      }
    }

    array_multisort($theme_options, SORT_NATURAL);
    return $theme_options;
  }

}
