<?php

declare(strict_types=1);

namespace Drupal\sites_group\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds a plugin for each configured group.
 */
class GroupSites extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * Constructs a GroupSites object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *  The entity type manager.
   */
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    // Get sites_group groups.
    // Injecting the sites_group.service causes plugin issues, so we load it
    // here statically.
    $sites_group_service = \Drupal::service('sites_group.service');
    foreach ($sites_group_service->getGroups() as $group_id => $group) {
      $this->derivatives[$group_id] = $base_plugin_definition;
      $this->derivatives[$group_id]['id'] = "sites_group:$group_id";
      $this->derivatives[$group_id]['title'] = $group->label();
      $this->derivatives[$group_id]['status'] = $group->isPublished();
      $this->derivatives[$group_id]['settings']['group_id'] = $group_id;

      // Caution: $group->getGroupType() uses the magic getter group->type,
      // which does not always work on cache rebuild.
      // Workaround: we load the group_type entity ourself.
      /** @var \Drupal\group\Entity\GroupTypeInterface $group_type */
      $group_type = $this->entityTypeManager->getStorage('group_type')->load($group->bundle());

      // Legacy hostname - alias settings
      if (!empty($this->derivatives[$group_id]['settings']['hostname'])) {
        $this->derivatives[$group_id]['settings']['hostnames'] = [$this->derivatives[$group_id]['settings']['hostname']];
      }
      if (isset($this->derivatives[$group_id]['settings']['hostname_aliases'])) {
        $this->derivatives[$group_id]['settings']['hostnames'] = array_merge(
          $this->derivatives[$group_id]['settings']['hostnames'],
          $this->derivatives[$group_id]['settings']['hostname_aliases'],
        );
      }

      // Get the values from the mapped fields and inject them as settings.
      $field_map = $group_type->getThirdPartySetting('sites_group', 'field_map', []);
      foreach ($field_map as $setting => $field_name) {
        if (!$group->get($field_name)->isEmpty()) {
          $main_property_name = $group->get($field_name)->first()->mainPropertyName();
          foreach ($group->get($field_name)->getValue() as $delta => $field_value) {
            // Map multi-value fields.
            if ($group->get($field_name)->count() > 1) {
              $this->derivatives[$group_id]['settings'][$setting][$delta] = $field_value[$main_property_name];
            }
            // Map single-value fields.
            else {
              $this->derivatives[$group_id]['settings'][$setting] = $field_value[$main_property_name];
            }
          }
        }
      }

      // Set default value fallbacks.
      $default_settings = $group_type->getThirdPartySetting('sites_group', 'default_settings', []);
      foreach ($default_settings as $setting => $default_setting_value) {
        if (empty($this->derivatives[$group_id]['settings'][$setting])) {
          $this->derivatives[$group_id]['settings'][$setting] = $default_setting_value;
        }
      }

      // Set the weight.
      if (isset($this->derivatives[$group_id]['settings']['weight'])) {
        $this->derivatives[$group_id]['weight'] = $this->derivatives[$group_id]['settings']['weight'];
      }
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
