<?php

declare(strict_types=1);

namespace Drupal\sites_group\Plugin\Site;

use Drupal\sites\Attribute\Site;
use Drupal\group\Entity\GroupInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\sites\Plugin\Site\SitePluginBase;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation for "group_sites_base".
 */
#[Site(
  id: "sites_group",
  title: new TranslatableMarkup('Group site'),
  description: new TranslatableMarkup('This site is provided by sites_group.'),
  weight: -2,
  deriver: "Drupal\sites_group\Plugin\Derivative\GroupSites"
)]
class GroupSitesBase extends SitePluginBase implements GroupSiteInterface {

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $route_match): ?bool {
    // Routes on the group belong to the group site.
    if ($group = $route_match->getParameter('group')) {
      if (!$group instanceof GroupInterface) {
        $group = \Drupal::entityTypeManager()->getStorage('group')->load($group);
      }

      return $group->id() == $this->getSetting('group_id');
    }

    // Restrict access to entities which don't belong to the group.
    $entity_type_manager = \Drupal::entityTypeManager();
    $entity_types = $entity_type_manager->getDefinitions();
    foreach ($entity_types as $entity_type_id => $entity_type) {
      if ($entity_type instanceof ContentEntityTypeInterface) {
        if ($entity = $route_match->getParameter($entity_type_id)) {
          // Remove this when https://www.drupal.org/project/drupal/issues/2528166 is in.
          if (is_scalar($entity)) {
            $entity = $entity_type_manager->getStorage($entity_type_id)->load($entity);
            if (!$entity) {
              return FALSE;
            }
          }
          $relationships = $entity_type_manager->getStorage('group_relationship')->loadByEntityAndGroup($entity, $this->getGroup());
          return !empty($relationships);
        }
      }
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  protected function getConfigOverrides(): array {
    $config = [];

    // Map the name setting to the site name.
    $config['system.site']['name'] = $this->getSetting('name');
    // Set group site as frontpage.
    $config['system.site']['page']['front'] = $this->getSetting('front_page') ?: '/group/' . $this->getSetting('group_id');

    if ($this->getSetting('403_page')) {
      $config['system.site']['page']['403'] = $this->getSetting('403_page');
    }

    if ($this->getSetting('404_page')) {
      $config['system.site']['page']['404'] = $this->getSetting('404_page');
    }

    return $config;
  }

  /**
   * {@inheritdoc}
   */
//  public function label(): string {
//    $label = parent::label();
//    return "{$this->getSetting('name')} ($label)";
//  }

  /**
   * {@inheritdoc}
   */
  public function getGroup(): ?GroupInterface {
    return \Drupal::entityTypeManager()->getStorage('group')->load($this->getSetting('group_id'));
  }

}
