<?php

declare(strict_types=1);

namespace Drupal\sites_group;

use Psr\Log\LoggerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\sites\Plugin\Site\SiteInterface;
use Drupal\sites\SitePluginManagerInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\sites_group\Plugin\Site\GroupSiteInterface;
use Drupal\sites\ContextProvider\CurrentSiteContextInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;

/**
 * The sites_group service.
 */
class SitesGroupService implements SitesGroupServiceInterface {

  use LoggerChannelTrait;

  /**
   * The logger which is logging to the "sites_group" channel. We are not
   * injecting the logger channel, like we would normally do, but using the
   * logger channel trait. This we are doing due to edge cases with sites_group
   * due to module installation.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private readonly LoggerInterface $logger;

  /**
   * Constructs a SitesGroupService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface $groupRelationTypeManager
   *   The group relation type manager.
   * @param \Drupal\sites\SitePluginManagerInterface $sitePluginManager
   *   The site plugin manager.
   */
  public function __construct(
    readonly public EntityTypeManagerInterface $entityTypeManager,
    readonly protected GroupRelationTypeManagerInterface $groupRelationTypeManager,
    readonly protected SitePluginManagerInterface $sitePluginManager,
    readonly public CurrentSiteContextInterface $currentSiteContext
  ) {
    $this->logger = $this->getLogger('sites_group');
  }

  /**
   * {@inheritdoc}
   */
  public function getGroupTypes(): array {
    $group_types = [];

    foreach ($this->entityTypeManager->getStorage('group_type')->loadMultiple() as $group_type) {
      if ($group_type->getThirdPartySetting('sites_group', 'status')) {
        $group_types[$group_type->id()] = $group_type;
      }
    }

    return $group_types;
  }

  /**
   * {@inheritdoc}
   */
  public function getGroups(): array {
    if (($group_types = $this->getGroupTypes()) && !empty($group_types)) {
      $group_storage = $this->entityTypeManager->getStorage('group');

      $entity_query = $group_storage->getQuery();
      $entity_query->accessCheck(FALSE);
      $entity_query->condition('type', array_keys($group_types), 'IN');
      $result = $entity_query->execute();

      return $result ? $group_storage->loadMultiple($result) : [];
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getGroupSiteEntityTypeIds(): array {
    $entity_type_ids = [];
    foreach ($this->getGroupTypes() as $group_type_id => $group_type) {
      foreach ($this->groupRelationTypeManager->getInstalled($group_type) as $install) {
        $entity_type_id = $install->getPluginDefinition()->get('entity_type_id');
        $entity_type_ids[$entity_type_id] = $entity_type_id;
      }
    }

    return $entity_type_ids;
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailableGroupSiteTypesByEntity(EntityInterface $entity): array {
    $group_types = [];
    foreach ($this->getGroupTypes() as $group_type_id => $group_type) {
      foreach ($this->groupRelationTypeManager->getInstalled($group_type) as $install) {
        if (!$this->relationTypeConnectsToEntity($install->getRelationType(), $entity)) {
          continue;
        }

        $group_types[$group_type_id] = $group_type;
      }
    }

    return $group_types;
  }

  /**
   * {@inheritdoc}
   */
  public function relationTypeConnectsToEntity(GroupRelationTypeInterface $group_relation_type, EntityInterface $entity): bool {
    if ($group_relation_type->getEntityTypeId() != $entity->getEntityTypeId()) {
      // The relation type does not support the entity's type.
      return FALSE;
    }
    if ($bundle = $group_relation_type->getEntityBundle()) {
      // The relation type supports a specfic bundle. Check if the bundle matches.
      return $bundle == $entity->bundle();
    }

    // The relation type supports all bundles.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function groupIsSite(GroupInterface $group): bool {
    return array_key_exists($group->bundle(), $this->getGroupTypes());
  }

  /**
   * {@inheritdoc}
   */
  public function getSiteForGroup(GroupInterface $group): ?SiteInterface {
    foreach ($this->sitePluginManager->getSites() as $site) {
      if ($site->getSetting('group_id') == $group->id()) {
        return $site;
      }
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getRelationship(EntityInterface $entity): ?GroupRelationshipInterface {
    // @todo Only overrideable entities.
    if ($entity && !$entity->isNew() && $site = \Drupal::service('sites.current_site')->getSiteFromContext()) {
      if (!$site instanceof GroupSiteInterface || !($group = $site->getGroup())) {
        return NULL;
      }

      $relationships = $this->entityTypeManager->getStorage('group_relationship')->loadByEntityAndGroup($entity, $group);
      if (count($relationships) == 1) {
        $relationship = reset($relationships);
        $langcode = $entity->language()->getId();
        if ($relationship->hasTranslation($langcode)) {
          return $relationship->getTranslation($langcode);
        }
      }
    }
    return NULL;
  }

  /**
   * Determines the group_relation_type plugin_id for a given entity.
   *
   * @todo Replace with GroupRelationTypeManager::getPluginIdsByEntityTypeId
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to check for.
   * @return string|null
   *   The plugin id.
   */
  public function getPluginIdByEntity(EntityInterface $entity): ?string {
    $group_relation_type_manager = \Drupal::service('group_relation_type.manager');
    $plugin_ids = $group_relation_type_manager->getPluginIdsByEntityTypeId($entity->getEntityTypeId());
    foreach ($plugin_ids as $plugin_id) {
      $group_relation = $group_relation_type_manager->getDefinition($plugin_id);
      if (!$this->relationTypeConnectsToEntity($group_relation, $entity)) {
        continue;
      }
      
      return $plugin_id;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isContentPluginInstalledForGroup(GroupInterface $group, string $pluginId): bool {
    $groupType = $group->getGroupType();
    $installedPlugins = $groupType->getInstalledPlugins();

    return $installedPlugins->has($pluginId);
  }

  /**
   * {@inheritdoc}
   */
  public function ensureSiteContext(ContentEntityInterface $entity): void {
    try {
      $site = $this->currentSiteContext->getSiteFromContext();
      if ($site instanceof GroupSiteInterface) {
        $group = $site->getGroup();
        $pluginId = $this->getPluginIdByEntity($entity);
        if ($group instanceof GroupInterface && $pluginId !== NULL) {
          if ($this->isContentPluginInstalledForGroup($group, $pluginId)) {
            $group->addRelationship(
              $entity,
              $this->getPluginIdByEntity($entity)
            );
          }
        }
      }
    } catch (\Throwable $t) {
      $this->logger->error($t->getMessage());
    }
  }

}
