<?php

namespace Drupal\sites_group;

use Drupal\Core\Entity\EntityInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\sites\Plugin\Site\SiteInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeInterface;

/**
 * The sites_group service interface.
 */
interface SitesGroupServiceInterface {

  /**
   * Loads the configured sites_group group types.
   *
   * @return \Drupal\group\Entity\GroupTypeInterface[]
   *   An array of available group types.
   */
  public function getGroupTypes(): array;

  /**
   * Loads the groups which should be used as sites.
   *
   * @return \Drupal\group\Entity\GroupInterface[]
   *   An array of groups.
   */
  public function getGroups(): array;

  /**
   * Determines all available entity_types installed as potential group content.
   *
   * @return array
   *   An array of entity_type id's.
   */
  public function getGroupSiteEntityTypeIds(): array;

  /**
   * Determines all sites_group types that use the given entity as content.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check for.
   *
   * @return \Drupal\group\Entity\GroupTypeInterface[]
   *   An array of group types.
   */
  public function getAvailableGroupSiteTypesByEntity(EntityInterface $entity): array;

  /**
   * Check whether an group relation type can connect to an entity.
   *
   * @param GroupRelationTypeInterface $group_relation_type
   *   The group relation type
   * @param EntityInterface $entity
   *   The entity
   * @return boolean
   *   TRUE, if the group relation type can connect to the entity.
   */
  public function relationTypeConnectsToEntity(GroupRelationTypeInterface $group_relation_type, EntityInterface $entity): bool;

  /**
   * Determines ig a group is a site.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to check for.
   *
   * @return bool
   *   TRUE if the group is used as site.
   */
  public function groupIsSite(GroupInterface $group): bool;

  /**
   * Loads a site plugin for a given group.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to look for.
   *
   * @return \Drupal\sites\Plugin\Site\SiteInterface|null
   *   The corresponding site plugin.
   */
  public function getSiteForGroup(GroupInterface $group): ?SiteInterface;

  /**
   * Get the group relationship of the given entity for the current site group.
   *
   * @param EntityInterface $entity
   *   The entity to get the corresponding relationship.
   * @return GroupRelationshipInterface|null
   *   The group relationship (if it exists).
   */
  public function getRelationship(EntityInterface $entity): ?GroupRelationshipInterface;

  /**
   * Checks if the content plugin is installed for the given group.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to check.
   * @param string $pluginId
   *   The plugin ID to check.
   *
   * @return bool
   *   TRUE if the content plugin is installed for the group, FALSE otherwise.
   */
  public function isContentPluginInstalledForGroup(GroupInterface $group, string $pluginId): bool;

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity type of type entity.
   *
   * @return void
   */
  public function ensureSiteContext(ContentEntityInterface $entity): void;

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for the interface.
   *
   * @return string|null
   */
  public function getPluginIdByEntity(EntityInterface $entity): ?string;

}
